#include <hmg.ch>
#include "hfcl.ch"

function ean8_barcode( cCode, cImageFile, nStripWidth, nStripHeight, lShowDigits )
   local cStartStop := '101'
   local aFirstPart := { '0001101',; 
                       '0011001',;
                       '0010011',;
                       '0111101',;
                       '0100011',;
                       '0110001',;
                       '0101111',;
                       '0111011',;
                       '0110111',;
                       '0001011' }      
   local aSecondPart := { '1110010',;
                        '1100110',;
                        '1101100',;
                        '1000010',;
                        '1011100',;
                        '1001110',;
                        '1010000',;
                        '1000100',;
                        '1001000',;
                        '1110100' }
   local cCenterPart :=  '01010'
   local nOddSum := 0
   local nEvenSum := 0
   local nCheckSum := 0
   local cBarCode := ''
   local hBitmap := nil
   local nFontSize := 10
   local nOffset := 0
   
   local nImageWidth := 0
   local nImageHeight := 0
   local BTStruct, hDC 
   
   default nStripWidth := 2
   default nStripHeight := 110
   default lShowDigits := .t.

   nOffset := 4 * nStripWidth
   nFontSize := 8 + ( nStripWidth * 2 )
   
   nImageWidth := ( nOffset + ( 67 * nStripWidth ) + nOffset )
   if lShowDigits
      nImageHeight := ( nOffSet + nStripHeight + nOffset + nFontSize + nOffSet )
   else
      nImageHeight := ( nOffSet + nStripHeight + nOffset )
   endif   
   
   cCode := alltrim( cCode )
   if len( cCode ) < 7 .or. len( cCode ) > 8
      msgstop( 'Invalid Code!' )
      return nil
   endif
   // find checksum
   nOddsum := val( substr( cCode, 1, 1 ) ) + ;
            val( substr( cCode, 3, 1 ) ) + ;
            val( substr( cCode, 5, 1 ) ) + ;
            val( substr( cCode, 7, 1 ) ) 
   nEvenSum := val( substr( cCode, 2, 1 ) ) + ;
            val( substr( cCode, 4, 1 ) ) + ;
            val( substr( cCode, 6, 1 ) )   
   nCheckSum := mod( ( 10 - mod( ( ( 3 * nOddSum ) + nEvenSum ), 10 ) ), 10 )

   do case
   case len( alltrim( cCode ) ) == 7
      cCode := cCode + str( nCheckSum, 1, 0 )
   case len( alltrim( cCode ) ) == 8
      if substr( cCode, 8, 1 ) <> str( nChecksum, 1, 0 )
         msgstop( 'Invalid Checksum! The last digit should be ' + str( nCheckSum, 1, 0 ) )
         return nil
      endif
   endcase
   
   // design barcode
   
   // Start
   cBarCode := cStartStop
   // First Part
   cBarCode := cBarCode + aFirstPart[ val( substr( cCode, 1, 1 ) ) + 1 ]
   cBarCode := cBarCode + aFirstPart[ val( substr( cCode, 2, 1 ) ) + 1 ]
   cBarCode := cBarCode + aFirstPart[ val( substr( cCode, 3, 1 ) ) + 1 ]
   cBarCode := cBarCode + aFirstPart[ val( substr( cCode, 4, 1 ) ) + 1 ]
   // CenterPart
   cBarCode := cBarCode + cCenterPart   
   // SecondPart   
   cBarCode := cBarCode + aSecondPart[ val( substr( cCode, 5, 1 ) ) + 1 ]
   cBarCode := cBarCode + aSecondPart[ val( substr( cCode, 6, 1 ) ) + 1 ]
   cBarCode := cBarCode + aSecondPart[ val( substr( cCode, 7, 1 ) ) + 1 ]
   cBarCode := cBarCode + aSecondPart[ val( substr( cCode, 8, 1 ) ) + 1 ]
   // Stop
   cBarCode := cBarCode + cStartStop
   
   hBitmap := BT_BitmapCreateNew ( nImageWidth, nImageHeight, { 255, 255, 255 } )
   hDC := BT_CreateDC ( hBitmap, BT_HDC_BITMAP, @BTstruct)
   
   nCurCol := nOffset
   
   
   for i := 1 to len( cBarCode )
      if substr( cBarCode, i, 1 ) == '1'
         BT_DrawFillRectangle( hDC, nOffset, nCurCol, nStripWidth, nStripHeight, { 0, 0, 0 } )
      endif   
      nCurCol := nCurCol + nStripWidth
   next i
   if lShowDigits   
      BT_DrawText( hDC, nOffSet + nStripHeight, nOffset, cCode, "Arial", nFontSize )   
   endif   
   BT_DeleteDC (BTstruct)
   BT_BitmapSaveFile( hBitmap, cImageFile, BT_FILEFORMAT_PNG )
   BT_BitmapRelease ( hBitmap )
return nil
